
{*******************************************************}
{                                                       }
{             Remote Wake Up  Version 1.0               }
{                                                       }
{         Copyright (c) 1999-2000 Vadim Crits           }
{                                                       }
{*******************************************************}

program rwu;

{$APPTYPE CONSOLE}

uses
  SysUtils, WinSock;

const
  LEN_PHYSADDR = 6;

var
  WSAData: TWSAData;
  MSocket: TSocket;
  SockAddrIn: TSockAddrIn;
  RetVal, OptVal, i: Integer;
  MacAddr: array[0..5] of Byte;
  MagicData: array[0..101] of Byte;
  Position: Longint;
  SAddr: string;

{$R VERSION.RES}

function CheckWinSockResult(ResultCode: Integer; const FuncName: string): Integer;
begin
     if ResultCode <> 0 then
     begin
          Result := WSAGetLastError;
          Writeln(Format(#13#10'Windows socket error: %s (%d), on API ''%s''.',
                        [SysErrorMessage(Result), Result, FuncName]));
          Halt;
     end else Result := 0;
end;

begin
     if ParamCount <> 1 then
     begin
          Writeln(#13#10'Remote Wake Up.'#13#10#13#10 +
                  'Usage: rwu ethernetaddress'#13#10#13#10 +
                  'Example: rwu 00-B0-D0-15-86-B4'#13#10#13#10 +
                  'This program is freeware.'#13#10 +
                  'Author: Vadim Crits'#13#10);
          Halt;
     end;
     try
        SAddr := Paramstr(1);
        if Length(SAddr) <> 17 then Abort;
        for i := 1 to Length(SAddr) do
            if i in [3, 6, 9, 12, 15] then
               if SAddr[i] <> '-' then Abort;
        for i := 0 to High(MacAddr) do
        begin
             if Pos('-', SAddr) > 0 then
                MacAddr[i] := StrToInt(HexDisplayPrefix + Copy(SAddr, 1, Pos('-', SAddr) - 1))
             else
                MacAddr[i] := StrToInt(HexDisplayPrefix + SAddr);
             Delete(SAddr, 1, Pos('-', SAddr));
        end;
     except
        Writeln(#13#10'Invalid Ethernet address.');
        Halt;
     end;
     CheckWinSockResult(WSAStartup($0101, WSAData), 'WSAStartup');
     MSocket := socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
     if MSocket = INVALID_SOCKET then CheckWinSockResult(MSocket, 'socket');
     OptVal := 1;
     CheckWinSockResult(setsockopt(MSocket, SOL_SOCKET, SO_BROADCAST,
                        PChar(@OptVal), SizeOf(OptVal)), 'setsockopt');
     FillChar(MagicData, SizeOf(MagicData), $FF);
     Position := LEN_PHYSADDR;
     while Position < SizeOf(MagicData) do
     begin
          Move(MacAddr, Pointer(Longint(@MagicData) + Position)^, LEN_PHYSADDR);
          Inc(Position, LEN_PHYSADDR);
     end;
     SockAddrIn.sin_family := AF_INET;
     SockAddrIn.sin_addr.S_addr := Longint(INADDR_BROADCAST);
     RetVal := sendto(MSocket, MagicData, SizeOf(MagicData), 0, SockAddrIn, SizeOf(SockAddrIn));
     if RetVal = SOCKET_ERROR then CheckWinSockResult(RetVal, 'sendto');
     CheckWinSockResult(closesocket(MSocket), 'closesocket');
     CheckWinSockResult(WSACleanup, 'WSACleanup');
end.
